
UiAbstractItemView subclass: #UiListViewNew
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

UiListViewNew subclass: #UiListIconView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiListIconView commentStamp: 'mt 10/14/2010 15:39:41.542' prior: 0!
I am a list view that is able to break the line of items in one direction and shows as many items as possible.!

!UiListIconView methodsFor: 'geometry' stamp: 'mt 10/14/2010 14:52:08.401'!
extent: aPoint

	super extent: aPoint.
	self viewport ifNotNil: [:v | v updateLayout].! !

!UiListIconView methodsFor: 'initialization' stamp: 'mt 10/14/2010 15:44:38.153'!
initialize

	super initialize.
	
	self defaultItemDelegate: UiIconicItemDelegate new.! !

!UiListIconView methodsFor: 'accessing' stamp: 'mt 10/14/2010 15:29:19.619'!
listOrientation: aSymbol
	"#vertical ... items go from left to right and break at width -> vertical scrolling
	#horizontal ... items go from top to bottom and break at height -> horizontal scrolling"

	self listOrientation = aSymbol
		ifTrue: [^ self].
		
	self viewport orientation: aSymbol.
	self updateLayout.! !

!UiListViewNew methodsFor: 'geometry' stamp: 'mt 10/14/2010 14:18:57.192'!
extent: aPoint

	super extent: aPoint.
	self viewport ifNotNil: [:v | v updateExtent].! !

!UiListViewNew methodsFor: 'model access' stamp: 'mt 10/14/2010 13:48:22.274'!
fillViewportFor: aModelIndex animated: aBoolean

	| index item |
	self model ifNil: [^ self].
	
	1 to: (self model rowCountFor: nil) do: [:row |
		index := self model indexRow: row column: 1 parent: nil.
		item := (self itemDelegateFor: index) itemFor: index.
		self viewport addMorph: item atIndex: index.
		aBoolean ifTrue: [item fadeIn]].! !

!UiListViewNew methodsFor: 'accessing - private' stamp: 'mt 10/14/2010 13:47:53.301'!
viewportClass

	^ UiListViewport! !

Object subclass: #UiViewInfo
	instanceVariableNames: 'size items resizeMode headerItem alignment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

UiViewInfo subclass: #UiViewColumnInfo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiViewColumnInfo commentStamp: 'mt 10/9/2010 11:38:45.048' prior: 0!
Useful index element for tree viewport.!

!UiViewColumnInfo methodsFor: 'initialization' stamp: 'mt 10/14/2010 15:51:51.099'!
initialize

	super initialize.
	self
		resize: 150;
		alignment: #top.! !

!UiViewColumnInfo methodsFor: 'layouting' stamp: 'mt 10/9/2010 19:58:18.468'!
layoutItemsRelativeTo: anItem withHeights: allHeights

	| itemFound offset |
	anItem = items last
		ifTrue: [^ self].
		
	itemFound := false.
	offset := allHeights first + 1.
	
	items withIndexDo: [:item :index |
		itemFound
			ifFalse: [itemFound := item == anItem]
			ifTrue: [
				item top: offset.
				offset := offset + (allHeights at: index)]]! !

!UiViewColumnInfo methodsFor: 'layouting' stamp: 'mt 10/9/2010 12:01:54.696'!
layoutItemsWithHeights: allHeights

	items ifEmpty: [^ self].
	self
		layoutItemsRelativeTo: items first
		withHeights: allHeights.! !

!UiViewInfo methodsFor: 'accessing' stamp: 'mt 10/14/2010 15:51:25.366'!
alignment

	^ alignment! !

!UiViewInfo methodsFor: 'accessing' stamp: 'mt 10/14/2010 15:51:34.721'!
alignment: aSymbol

	alignment := aSymbol.! !

!UiViewRowInfo methodsFor: 'initialization' stamp: 'mt 10/14/2010 15:52:09.383'!
initialize

	super initialize.
	self
		resize: Preferences standardListFont height;
		alignment: #left.! !

!UiViewRowInfo methodsFor: 'layouting' stamp: 'mt 10/9/2010 20:10:58.793'!
layoutItemsRelativeTo: anItem withWidths: allWidths

	| itemFound offset |
	anItem = items last
		ifTrue: [^ self].
		
	itemFound := false.
	offset := allWidths first + 1.
	items withIndexDo: [:item :index |
		itemFound
			ifFalse: [itemFound := item == anItem]
			ifTrue: [
				item left: offset.
				offset := offset + (allWidths at: index)]]! !

!UiViewRowInfo methodsFor: 'layouting' stamp: 'mt 10/9/2010 18:27:22.1'!
layoutItemsWithWidths: allWidths

	items ifEmpty: [^ self].
	self
		layoutItemsRelativeTo: items first
		withWidths: allWidths.! !

!UiListIconViewport methodsFor: 'as yet unclassified' stamp: 'mt 10/14/2010 16:00:39.347'!
addRowFor: aModelIndex

	| row |
	row := super addRowFor: aModelIndex.
	row alignment: #middle.
	^ row.! !

!UiListIconViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 10/14/2010 15:01:39.569'!
cellBoundsFor: aCellIndex

	^ (((aCellIndex - (1@1)) * maxItemExtent) + self topLeft)
		extent: maxItemExtent! !

!UiListIconViewport methodsFor: 'rows/columns - to cell' stamp: 'mt 10/14/2010 15:25:39.83'!
cellIndexFromPoint: aPoint
	"Given a point in pixel coordinates."
	
	| cell |
	cell := ((aPoint - self topLeft) // (maxItemExtent)) + (1@1).
	
	self orientation = #vertical
		ifTrue: [
			(cell x > self maxItemsPerLine
				or: [cell y - 1 * self maxItemsPerLine + cell x > rows size])
				ifTrue: [^ nil]]
		ifFalse: [
			(cell y > self maxItemsPerLine
				or: [cell x - 1 * self maxItemsPerLine + cell y > rows size])
				ifTrue: [^ nil]].
	
	^ cell! !

!UiListIconViewport methodsFor: 'drawing' stamp: 'mt 10/14/2010 14:50:09.605'!
drawSubmorphsOn: aCanvas 
	
	submorphs isEmpty ifTrue: [^self].
	aCanvas
		clipBy: self clippingBounds
		during: [:canvas | submorphs reverseDo: [:m | canvas fullDrawMorph: m]]! !

!UiListIconViewport methodsFor: 'initialization' stamp: 'mt 10/14/2010 14:30:29.463'!
initialize

	super initialize.
	self selectionBehavior: #selectCells.! !

!UiListIconViewport methodsFor: 'rows/columns - to model index' stamp: 'mt 10/14/2010 15:38:35.676'!
modelIndexFor: aCellIndex

	aCellIndex ifNil: [^ nil].
	
	^ self orientation = #vertical
		ifTrue: [rows at: aCellIndex y - 1 * self maxItemsPerLine + aCellIndex x]
		ifFalse: [rows at: aCellIndex x - 1 * self maxItemsPerLine + aCellIndex y]! !

!UiListIconViewport methodsFor: 'accessing' stamp: 'mt 10/14/2010 15:16:30.729'!
orientation

	^ orientation ifNil: [orientation := #vertical]! !

!UiListIconViewport methodsFor: 'accessing' stamp: 'mt 10/14/2010 15:28:54.804'!
orientation: aSymbol
	"#vertical ... items go from left to right and break at width -> vertical scrolling
	#horizontal ... items go from top to bottom and break at height -> horizontal scrolling"

	orientation := aSymbol.! !

!UiListIconViewport methodsFor: 'geometry' stamp: 'mt 10/14/2010 15:33:51.365'!
updateExtent

	| sBounds |
	sBounds := (self submorphBounds extent / maxItemExtent) ceiling * maxItemExtent.
	self extent: 
		(sBounds max: (self owner extent - 5))! !

!UiListIconViewport methodsFor: 'layouting' stamp: 'mt 10/14/2010 15:59:50.261'!
updateLayout

	| offset |
	maxItemExtent := 1@1.
	columns first items do: [:item | maxItemExtent := maxItemExtent max: item extent].

	offset := self topLeft.
	self rowsDo: [:rowInfo | | item |
		item := rowInfo items first.
		
		rowInfo alignment = #left
			ifTrue: [item topLeft: offset].
		rowInfo alignment = #middle
			ifTrue: [item topLeft: offset + ((maxItemExtent x - item width)//2 @0)].
		rowInfo alignment = #right
			ifTrue: [item topRight: offset + (maxItemExtent x @ 0)].

		self orientation = #vertical
			ifTrue: [
				offset := offset + (maxItemExtent x @ 0).
				(offset x + maxItemExtent x)  > (self owner width - 5)
					ifTrue: [offset := self left @ (offset y + maxItemExtent y)]]
			ifFalse: [
				offset := offset + (0 @ maxItemExtent y).
				(offset y + maxItemExtent y)  > (self owner height - 5)
					ifTrue: [offset := (offset x + maxItemExtent x) @ self top]]].
		
	self updateExtent.		! !

UiAbstractItemDelegate subclass: #UiSimpleItemDelegate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiSimpleItemDelegate methodsFor: 'building' stamp: 'mt 10/13/2010 14:42:42.25'!
itemFor: aModelIndex

	| icon text item offset textMorph |
	icon := aModelIndex icon "ifNil: [Morph new extent: 16@16; color: Color transparent]".
	text := (aModelIndex text ifNil: [' ']) asText.
	
	textMorph := text asMorph lock.
	
	item := Morph new
		color: Color transparent;
		height: textMorph height. "for smaller items use font height"
	
	icon ifNotNil: [item addMorphBack: icon asMorph].
	item addMorphBack: textMorph.
	
	"Store morphs in properties for later access."
	item
		setProperty: #text toValue: text;
		setProperty: #icon toValue: icon.
	
	"Manual layouting."
	offset := item left + 3.
	item submorphs do: [:m |
		m left: offset.
		offset := offset + m width + 2.
		m top: (item top - (m height - item height // 2))].
	
	"Expand item horizontally to its contents."
	item
		width: item submorphBounds width + 10; "font rendering bugfix"
		height: item height + 1. "font rendering bugfix"
		
	^ item! !

!UiItemViewport methodsFor: 'rows/columns - to info' stamp: 'mt 10/11/2010 16:18:23.881'!
columnAt: index

	^ columns
		at: index
		ifAbsentPut: [UiViewColumnInfo new]! !

!UiItemViewport methodsFor: 'layouting' stamp: 'mt 10/14/2010 13:07:53.401'!
updateLayout

	| heights widths |
	columns do: [:column | column updateSize].
	rows do: [:row | row updateSize].

	heights := rows collect: [:r | r height].
	widths := columns collect: [:c | c width].
	
	columns do: [:column | column layoutItemsWithHeights: heights].
	rows do: [:row | row layoutItemsWithWidths: widths].
	
	columns first items do: [:item |
		item left: self left + (item valueOfProperty: #indent)].

	"Relayout changed header items."
	self horizontalHeader ifNotNil: [:hh |
		hh removeAllMorphs.
		columns do: [:colInfo | hh addMorphBack: colInfo headerItem]].
	self verticalHeader ifNotNil: [:vh |
		vh removeAllMorphs.
		rows do: [:rowInfo | vh addMorphBack: rowInfo headerItem]].
	
	"Shrink wrap to contents."
	self updateExtent.! !

UiItemViewport subclass: #UiListViewport
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiListViewport methodsFor: 'rows/columns - to cell' stamp: 'mt 10/14/2010 14:25:48.747'!
cellIndexFromPoint: aPoint
	"Assume there is only 1 column."
	
	| offset |
	offset := self topLeft.
	rows withIndexDo: [:rowInfo :rowIndex | 
		((offset extent: self width @ rowInfo height) containsPoint: aPoint)
			ifTrue: [^ 1 @ rowIndex].
		offset := offset translateBy: 0 @ rowInfo height].
	
	^ nil! !

!UiListViewport methodsFor: 'geometry' stamp: 'mt 10/14/2010 14:06:17.833'!
initialize

	super initialize.
	self selectionBehavior: #selectRows.! !

!UiListViewport methodsFor: 'geometry' stamp: 'mt 10/14/2010 14:21:13.617'!
updateExtent

	| newHeight optimalWidth |	
	newHeight := rows
		ifEmpty: [0]
		ifNotEmpty: [(rows collect: [:row | row height]) sum].
	
	optimalWidth := columns first optimalSize.
	
	optimalWidth < self owner width
		ifTrue: [self extent: self owner width - 5 @ newHeight]
		ifFalse: [self extent: optimalWidth @ newHeight].
! !
